package main

import (
	"context"
	"flag"
	"log"
	"net"
	"net/http"

	gw "git.alfi.li/gamelang/proto_gorm/proto/gen/go" // Update
	"github.com/golang/glog"
	"github.com/grpc-ecosystem/grpc-gateway/v2/runtime"
	"github.com/jinzhu/gorm"
	_ "github.com/jinzhu/gorm/dialects/sqlite"
	"google.golang.org/grpc"
)

var (
	// command-line options:
	// gRPC server endpoint
	grpcServerEndpoint = flag.String("grpc-server-endpoint", "localhost:9090", "gRPC server endpoint")
	httpServerEndpoint = flag.String("http-server-endpoint", "0.0.0.0:8081", "http server endpoint")
)

func run() error {
	ctx := context.Background()
	ctx, cancel := context.WithCancel(ctx)
	defer cancel()

	// init GORM DB Server
	db, err := gorm.Open("sqlite3", "test.db")
	if err != nil {
		panic("failed to connect database")
	}
	defer db.Close()
	dbC := gw.IntPointServiceDefaultServer{}
	dbC.DB = db
	log.Println("database initialized")

	// start gRPC Server
	lis, err := net.Listen("tcp", *grpcServerEndpoint)
	if err != nil {
		log.Fatalf("failed to listen: %v", err)
	}
	var serverOpts []grpc.ServerOption
	grpcServer := grpc.NewServer(serverOpts...)
	gw.RegisterIntPointServiceServer(grpcServer, NewIntPointServiceServer(&dbC))
	go grpcServer.Serve(lis)
	log.Println("grpc server started on", *grpcServerEndpoint)

	// start REST Gateway
	mux := runtime.NewServeMux()
	dialOpts := []grpc.DialOption{grpc.WithInsecure()}
	err = gw.RegisterIntPointServiceHandlerFromEndpoint(ctx, mux, *grpcServerEndpoint, dialOpts)
	if err != nil {
		return err
	}

	// init WesocketGateway
	RegisterWebsocketHandler(ctx, mux, *grpcServerEndpoint, dialOpts)

	// Start HTTP server (and proxy calls to gRPC server endpoint)
	log.Println("http listening on", *httpServerEndpoint)
	return http.ListenAndServe(*httpServerEndpoint, mux)
}

func main() {
	flag.Parse()
	defer glog.Flush()

	if err := run(); err != nil {
		glog.Fatal(err)
	}
}
